<?php
namespace Drupal\templating;

use Drupal\Core\Url;
use Drupal\Core\Render\Markup;

class EntityInlineTemplate
{
    public function getRegionList(){
        $result = [];
        $config_settings = \Drupal::config("template_inline.settings") ;
        $allowed_theme = $config_settings->get('theme');
        $allowed_theme = array_values($allowed_theme);
        foreach ($allowed_theme as $theme){
            if(!is_numeric($theme)){
              $system_region = system_region_list($theme, $show = REGIONS_ALL);
              foreach ($system_region as $key => $region){
                    $result[$key] = $key ;
              }
            }
        }
        return $result ;
    }
    public  static  function  getModeViewList($entity_name){
        $mode_view_list = [] ;
        $mode_views = \Drupal::entityTypeManager()->getStorage('entity_view_mode')->loadMultiple();
        foreach ($mode_views as $key => $item){
            $type = $item->getTargetType();
            if($type == $entity_name){
                $mode_view_list[str_replace($type.'.', '', $key)] = str_replace($type.'.', '', $key);
            }
        }
        return $mode_view_list ;
    }
         /**
      *  template.[ENTITY_NAME].[THEME].[BUNDLE].[FIELD_NAME].[MODE_VIEW]
     **/
     function getTemplateField($variables){
        $bundle = trim($variables['element']['#bundle']);
        $field_name = trim($variables['element']['#field_name']);
        $entity_type = trim($variables['element']['#entity_type']); 
        $config = null ;       
        $theme = $this->is_allowed();
        if(!$theme){
        //    return false;
        }
        $activeThemeName = \Drupal::service('theme.manager')->getActiveTheme();
        $theme = $activeThemeName->getName();
        $suggestion_1 = $this->formatName('template.field--'.$theme.'-'.$entity_type.'-'.$bundle.'-'.$field_name.'.html.twig');
        $output = false ;
        $config_id = \Drupal::config($suggestion_1) ;
        if($config_id && $config_id->get('content')){
            $config = $config_id;
        }
        if($config && $config->get('content') &&
            $config->get('status')
        ){
            $output =  $config->get('content');
        }
        return $this->assetInjection($output,$config) ;
     }
     /**
      *  template.[ENTITY_NAME].[THEME].[BUNDLE].[MODE_VIEW]
     **/
     function getTemplateBlock($variables){
         $plugin_id = trim($variables['elements']['#plugin_id']);
         $base_plugin_id = trim($variables['elements']['#base_plugin_id']);
         $theme = $this->is_allowed();
         if(!$theme){
             return false;
         }
         $suggestion_1 = $this->formatName('template.block--'.$theme.'-'.$plugin_id.'.html.twig');
         $suggestion_2 = $this->formatName('template.block--'.$theme.'-'.$base_plugin_id.'.html.twig');
         $output = false ;
         $config = \Drupal::config($suggestion_2) ;
         $config_id = \Drupal::config($suggestion_1) ;
         if($config_id && $config_id->get('content')){
             $config = $config_id;
         }
         if($config && $config->get('content') &&
             $config->get('status')
         ){
             $output =  $config->get('content');
         }
         return $output ;
     }
     function formatName($name){
         return str_replace('_', '-', $name);
     }
    function removePrefix($name){
        return str_replace('template.', '', $name);
    }
    function generateTemplateLinkBlockContent($block,$mode_view){
        $activeThemeName = \Drupal::service('theme.manager')->getActiveTheme();
        $theme = $activeThemeName->getName();
        $bundle = $block->bundle();
        $id = $block->id();
        $suggestion = $this->formatName('template.block--'.$theme.'-'.$bundle."-".$mode_view.".html.twig");
        $suggestion_id = $this->formatName('template.block--'.$theme.'-'.$bundle."-".$mode_view."-".$id.".html.twig");
        $config = \Drupal::config($suggestion) ;
        $config_id = \Drupal::config($suggestion_id) ;
        if($config_id && $config_id->get('content')){
            $config = $config_id;
            $suggestion = $suggestion_id ;
        }
        if($config  && $config->get('content')){
           return $suggestion ;
        }
        return false ;
    }
    function getTemplateBlockContent($variables){
        $config = null; 

        $theme = $this->is_allowed();
        if(!$theme){
            return false;
        }
        $enabled_themes = \Drupal::service('theme_handler')->listInfo();
        $themebase = \Drupal::service('theme_handler')->getBaseThemes($enabled_themes, $theme );
        $base = null; 
        if(!empty($themebase)){
            $base = array_keys($themebase)[0] ;
        }
         $content_block = isset($variables['content']['#block_content']) ? $variables['content']['#block_content'] : $variables['content']['content']['#block_content'];
         $mode_view = isset($variables['content']['#view_mode'])? $variables['content']['#view_mode'] : $variables['content']['content']['#view_mode'];
         $bundle = $content_block->bundle();
         $id = $content_block->id();
         $suggestion = $this->formatName('template.block--'.$theme.'-'.$bundle."-".$mode_view.".html.twig");
         $suggestion_id = $this->formatName('template.block--'.$theme.'-'.$bundle."-".$mode_view."-".$id.".html.twig");
         $output = false ;
         $config_current = \Drupal::config($suggestion) ;
         $config_id = \Drupal::config($suggestion_id) ;
         // base theme 
         if($base){
            $suggestionbase = $this->formatName('template.block--'.$base.'-'.$bundle."-".$mode_view.".html.twig");
            $config_id_base = \Drupal::config($suggestionbase) ;
            if($config_id_base && $config_id_base->get('content') && 
            $config_id_base->get('status')){
               $config = $config_id_base;
            }
         }
         // current theme
         if($config_current && $config_current->get('content')){
            $config = $config_current;
          }

         // current theme by id
         if($config_id && $config_id->get('content')){
             $config = $config_id;
         }


         if($config && $config->get('content') &&
             $config->get('status')
         ){
            
             $output =  $config->get('content');
             $output = $this->injectionSpacer($output) ;
             $output = $this->assetInjection($output,$config) ;
         }
         return $output ;
     }
     function injectionSpacer($output){
        return  "{{spacer_top(content)|raw}}".$output."{{spacer_bottom(content)|raw}}" ;
     }
     function assetInjection($output,$config){
        //  if($config){
        //     if($config->get('css') && $config->get('css')!=''){
        //         $output = '<style>'.$config->get('css').'</style>'.$output;
        //     } 
        //     if($config->get('js') && $config->get('js')!=''){
        //         $output = $output.'<script>'.$config->get('js').'</script>';
        //     } 
        //  }             
        return $output;
     }
     function getAllAsset(){
        $results =\Drupal::configFactory()->listAll("template.");   
        $asset["css"] = "";
        $asset["js"] = "";
        $theme = \Drupal::theme()->getActiveTheme();
      //  $config = \Drupal::config('system.theme');
      //  $default_theme =  $config->get('default');
       // $current_theme =  $theme->getName();
        $activeThemeName = \Drupal::service('theme.manager')->getActiveTheme();

        $default_theme = \Drupal::service('theme_handler')->getDefault();
     
        $enabled_themes = \Drupal::service('theme_handler')->listInfo();
      
        $themebase = \Drupal::service('theme_handler')->getBaseThemes($enabled_themes, $default_theme);


       // $themebase = $activeThemeName->getBaseThemes();
        $base = null; 
        if(!empty($themebase)){
            $base = array_keys($themebase)[0] ;
        }

        foreach ($results as $key => $result) {
            $config = \Drupal::config($result) ;
            if($config && $config->get('content') 
            && $config->get('css') && $config->get('css')!='' &&
            $config->get('status')){
                if($config->get('theme') == $current_theme ) {
                   $asset["css"] = $this->minify(" /* " .$result. " */ " .$asset["css"] .$config->get('css')) ;
                } 
                if($config->get('theme') == $base ) {
                   $asset["css"] =  $this->minify(" /* " .$result. " */ " .$asset["css"] .$config->get('css')) ;
                }             
            }
            if($config && $config->get('content') 
            && $config->get('js') && $config->get('js')!='' &&
            $config->get('status')){
                if($config->get('theme') == $current_theme ) {
                  $asset["js"] =  $this->minify(" /* " .$result. " */ ".$asset["js"] .$config->get('js'),'js') ;
                }
                if($config->get('theme') == $base ) {
                  $asset["js"] =  $this->minify(" /* " .$result. " */ ".$asset["js"] .$config->get('js'),'js') ;
                }
            }
             
        }
        return $asset;
     }
     function is_allowed(){
        $current_theme = \Drupal::theme()->getActiveTheme();
        $theme = $current_theme->getName();      
        $config_settings = \Drupal::config("template_inline.settings") ;
       // $allowed_theme = $config_settings->get('theme');
       // $allowed_theme = array_values($allowed_theme);
       // $allowed_theme = array_map('strval',  $allowed_theme);
        $enabled = $config_settings->get('enabled');
        if(!$enabled){
            return false ;
        }
       // if(!in_array($theme , $allowed_theme) ){
       //    return false ;
       // }
        return $theme ;       
     }

     function getTemplatePage(&$variables){
        $theme = $this->is_allowed();
        if(!$theme){
            return false;
        }
        $current_route = \Drupal::routeMatch();
        $route_name = $current_route->getRouteName();   
        $entity_name = $variables['page']['#type'];
        foreach($variables['page'] as $key => $region){
            if(!str_starts_with($key, '#')) {
                $suggestion_1 = $this->formatName("template.".$entity_name."__".$theme."_".$route_name."_".$key.".html.twig") ;
                $output = false ;
                $config = \Drupal::config($suggestion_1) ;
                if($config && $config->get('content') &&
                    $config->get('status')
                ){
                    $output =  $config->get('content');
                }
                if($output){
                  $element = [
                    '#type' => 'inline_template',
                    '#template' => $output,
                    '#context' => [
                        'content' =>  $variables['page'][$key]
                    ]
                  ];
                  $variables['page'][$key] = $element;
                }
            }
        }
     }
     function getTemplateNode($variables){
        $theme = $this->is_allowed();
        if(!$theme){
            return false;
        }
        
         $entity_name = $variables['elements']['#entity_type'];
         $entity = $variables['elements']['#'.$entity_name] ;
         $view_mode = $variables['elements']['#view_mode'] ;
         $bundle = $entity->bundle();
         $id = $entity->id();
         
        $output = false ;
     
        // base theme  
        $activeThemeName = \Drupal::service('theme.manager')->getActiveTheme();
        $themebase = $activeThemeName->getBaseThemes();
        $base = null; 
         if(!empty($themebase)){
             $base = array_keys($themebase)[0] ;
         }
         if($base){
            $suggestionbase = $this->formatName("template.".$entity_name."__".$base."_".$bundle."_".$view_mode.".html.twig") ;   
            $config_base = \Drupal::config($suggestionbase) ;
            if($config_base && $config_base->get('content') && 
            $config_base->get('status')){
               $config = $config_base;
            }
         }
        // theme 
        $suggestion_1 = $this->formatName("template.".$entity_name."__".$theme."_".$bundle."_".$view_mode.".html.twig") ;
        $config_current = \Drupal::config($suggestion_1) ;
        if($config_current && $config_current->get('content')){
                $config = $config_current;
        }
        // theme by id  
        $suggestion_2 = $this->formatName("template.".$entity_name."__".$theme."_".$bundle."_".$view_mode."_".$id.".html.twig") ;
        $config_id = \Drupal::config($suggestion_2) ;
        if($config_id && $config_id->get('content')){
             $config = $config_id;
        }

        if($config && $config->get('content') &&
            $config->get('status')
        ){
            $output =  $config->get('content');
            $output = $this->assetInjection($output,$config) ;
        }
        return $output ;
    }
    function getTemplateEntity($variables){
        $theme = $this->is_allowed();
        if(!$theme){
            return false;
        }
        
         $entity_name = $variables['element']['#entity_type'];
         $entity = $variables['element']['#'.$entity_name] ;
         $view_mode = $variables['element']['#view_mode'] ;
         $bundle = $entity->bundle();
         $id = $entity->id();
         
        $output = false ;
        $suggestion_1 = $this->formatName("template.".$entity_name."__".$theme."_".$bundle."_".$view_mode.".html.twig") ;
        $suggestion_2 = $this->formatName("template.".$entity_name."__".$theme."_".$bundle."_".$view_mode."_".$id.".html.twig") ;

        $config = \Drupal::config($suggestion_1) ;
        $config_id = \Drupal::config($suggestion_2) ;
        if($config_id && $config_id->get('content')){
             $config = $config_id;
        }
        if($config && $config->get('content') &&
            $config->get('status')
        ){
            $output =  $config->get('content');
            $output =  $this->assetInjection($output,$config) ;
        }
        return $output ;
    }
     function getResults($output,$key,$result,$params){
         $operations = [] ;
         $config = \Drupal::config($result) ;
         $status =  $config->get('status');
         $bundle =  ($config->get('bundle'))? $config->get('bundle') : "";
         $operations['edit'] = array(
             'title' => t('Edit'),
             'url' => Url::fromRoute('templating.editor', array('config'=>$result))
         );
//          $operations['updateFile'] = array(
//              'title' => $this->t('update File Yml'),
//              'url' => Url::fromRoute('templating.manager', array('updateFile' => $result))
//          );
         $operations['remove'] = array(
             'title' => t('Remove'),
             'url' => Url::fromRoute('templating.manager', array('delete' => $result))
         );
//          $operations['removeFile'] = array(
//              'title' => $this->t('Remove Yml'),
//              'url' => Url::fromRoute('templating.manager', array('deleteFile' => $result))
//          );
         if(isset($params['key']) && $params['key'] !=""){
             if (is_string($params['key']) && strpos($result, $params['key']) !== false) {
                 $output[$result] = [
                     'id' => $key+1 ,
                     'config_name' =>  $this->renderName($result),
                     'bundle' =>    $bundle ,
                     'status' =>  ($status == 1) ? 'Enable':'Disable',
                     'operation' => array('data' => array('#type' => 'operations', '#links' => $operations)),
                 ];
             }
         }else{
             $output[$result] = [
                 'id' => $key+1 ,
                 'config_name' =>  $this->renderName($result),
                 'bundle' =>    $bundle ,
                 'status' =>  ($status == 1) ? 'Enable':'Disable',
                 'operation' => array('data' => array('#type' => 'operations', '#links' => $operations)),
             ];
         }
         return $output ;

     }
     function getThemeList(){
         $list = \Drupal::service('extension.list.theme')->getList();
         $themes = [];
         foreach ($list as $theme){
             if(is_object($theme)){
                 $path = $theme->getPath();
                 $path_array = explode('/',$path);
                 if(!empty($path_array)
                     &&$path_array[0]
                     && $path_array[1]
                     && $path_array[0] =='themes' && $path_array[1] =='custom'
                     && $theme->status == 1
                 ){
                     $themes[$theme->getName()] = [
                         "name"=>$theme->getName(),
                         "path"=> $theme->getPath()
                     ];
                 }
             }
         }
         return $themes ;

     }
     function getThemePath($item){
         $config = \Drupal::config($item) ;
         $theme_name = $config->get('theme') ;
         $list = \Drupal::service('extension.list.theme')->getList();
         if(in_array($theme_name,$list)){
             return \Drupal::service('extension.list.theme')->getPath($theme_name);
         }else{
             return false ;
         }

     }
     function getConfigRootPath(){
         return '/config/templates' ;
     }

    
     function deploy($item){
         $status = false ;
         $config = \Drupal::config($item) ;
         $theme_name = $config->get('theme') ;
         $content = $this->preformatContent($item);
         $theme = \Drupal::service('extension.list.theme')->getPath($theme_name);
         $isAllowed = $this->isAllowed($theme);
         if($isAllowed){
             $path_location = DRUPAL_ROOT."/".$theme."/templates/templating";
             if (!is_dir($path_location)) {
                $fileSystem = \Drupal::service('file_system');
                if ($fileSystem->mkdir($path_location, 0777, TRUE) === FALSE) {
                    drupal_set_message(t('Failed to create directory ' . $path_location), 'error');
                    return FALSE;
                }
            }else{
                @chmod($path_location  , 0777);
             }
             $file_name = $this->removePrefix($item);
             $status = $this->generateFile($path_location,$file_name,$content) ;
         }else{
             \Drupal::messenger()->addMessage(t('Failed to deploy template in theme ' . $theme), 'error');
         }
         $name = $this->removePrefix($item);
         if($status){
             \Drupal::messenger()->addMessage(t('Sucess to deploy template  ' . $name), 'status');
         }else{
             \Drupal::messenger()->addMessage(t('Failed to deploy template  ' . $name), 'error');
         }
         return false ;
     }
     function preformatContent($item){
        $config = \Drupal::config($item) ;
        $content = $config->get('content') ;
        $type = $config->get('type');
        $type = ($type == 'block_content')? 'block': $type ;
        $module_handler = \Drupal::service('module_handler');
        $path_module =  $module_handler->getModule('templating')->getPath();
        $path_location = DRUPAL_ROOT."/".$path_module."/templates/template-".$type.".html.twig.txt";
        $file_content = file_get_contents($path_location);
      
        $file_content = str_replace('{{'.$type.'_base_templating}}',$content,$file_content);
        return $file_content ;
     }
    function isAllowed($path_theme){
        $path_array = explode('/',$path_theme);
        if(!empty($path_array)
            &&$path_array[0]
            && $path_array[1]
            && $path_array[0] =='themes' && $path_array[1] =='custom'
        ){
            return true ;
        }else{
            return false ;
        }


    }
    function generateFile($directory, $filename, $content)
    {
        $fileSystem = \Drupal::service('file_system');
        if (!is_dir($directory)) {
            if ($fileSystem->mkdir($directory, 0777, TRUE) === FALSE) {
                \Drupal::messenger()->addMessage(t('Failed to create directory ' . $directory), 'error');
                return FALSE;
            }
        }
        if (!@chmod($directory . '/' . $filename , 0777)) {
               \Drupal::messenger()->addMessage(t('Failed to change permission file ' . $filename), 'error');
        }
        if (file_put_contents($directory . '/' . $filename , $content) === FALSE) {
            \Drupal::messenger()->addMessage(t('Failed to write file ' . $filename), 'error');
            return FALSE;
        }
        if (@chmod($directory . '/' . $filename , 0777)) {
         //   \Drupal::messenger()->addMessage(t('Failed to change permission file ' . $filename), 'error');
        }
        return TRUE;
    }
    function renderName($config_name){
        $name = $this->removePrefix($config_name);
        $new = Markup::create( $name.' ( <span style="color:red"> new </span> )');
        $diff = Markup::create($name.' ( <a href="#"><span style="color:blue"> exist </span></a> ) ');
        return  ($this->isExistLocal($config_name))? $diff : $new ;
    }
    public function isExistLocal($config_name){
        $path = $this->getConfigRootPath();
        $element = DRUPAL_ROOT.$path.'/'.$config_name.'.yml';
        if (file_exists($element)) {
            return true ;
        }
        return false ;
    }
    public  function searchFileInDirectory($key,$directory)
    {
        $path_file = [];
        if (is_dir($directory)) {
            $it = scandir($directory);
            if (!empty($it)) {
                foreach ($it as $fileinfo) {
                    $element =  $directory . "/" . $fileinfo;
                    if (is_dir( $element ) && substr($fileinfo, 0, strlen('.')) !== '.') {
                        $childs = $this->searchFileInDirectory($key,$element);
                        $path_file = array_merge($childs , $path_file);
                    }else{
                        if ($fileinfo && basename($fileinfo) == $key) {
                            if (file_exists($element)) {
                                $path_file[$key] = $element ;
                            }
                        }
                    }
                }
            }
        }else{
            \Drupal::messenger()->addMessage(t('No permission to read directory ' . $directory ), 'error');
            @chmod($directory  , 0777);
        }
        return $path_file;
    }
    public function minify($item,$type = "css")
    {
       if($type == "css"){
         return $this->minifyCSS($item) ;
       }
       if($type == "js"){
        return $this->minifyJS($item) ;
       }
       return $item ;
    }
    public function minifyCSS($css)
    {
      $css = preg_replace('/\/\*((?!\*\/).)*\*\//', '', $css); // negative look ahead
      $css = preg_replace('/\s{2,}/', ' ', $css);
      $css = preg_replace('/\s*([:;{}])\s*/', '$1', $css);
      $css = preg_replace('/;}/', '}', $css);
      return $css; 
    }
    function minifyJS($javascript){
        return preg_replace(array("/\s+\n/", "/\n\s+/", "/ +/"), array("\n", "\n ", " "),
        $javascript);
    }

}