<?php

namespace Drupal\templating\Form;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Edit config variable form.
 */
class ConfigTemplateEditor extends FormBase
{

    /**
     * {@inheritdoc}
     */
    public function getFormId()
    {
        return 'config_template_edit_form';
    }
    public function processContexualLiknk()
    {
        $query = $this->getRequest()->query;
        $config_json = $query->get('config_json');
        $param = null;
        if ($config_json) {
            $bid = json_decode($config_json);
            if ($bid && isset($bid->layout_builder_block)) {
                $current_theme = $bid->theme->current;
                $route_parameters = $bid->layout_builder_block->route_parameters;
                $uuid = $route_parameters->uuid;
                $delta = $route_parameters->delta;
                $section_storage = $route_parameters->section_storage;
                $section_storage_array = explode('.', $section_storage);
                $node_object = \Drupal::entityTypeManager()->getStorage($section_storage_array[0])->load($section_storage_array[1]);
                $config = $node_object->layout_builder__layout;
                $config = $config->getSection($delta);
                $config = $config->getComponent($uuid);
                $config = $config->getPlugin();
                $config = $config->getConfiguration();

                $mode_view = $config['view_mode'];
                $bundle = explode(":", $config['id'])[1];

                $services = \Drupal::service('templating.manager');
                $config_template_name = $services->formatName('template.block--' . $current_theme . '-' . $bundle . "-" . $mode_view . ".html.twig");
                $config_template = \Drupal::config($config_template_name);
                if ($config_template && $config_template->get('content') &&
                    $config_template->get('status')) {
                    $param = $config_template_name;
                } else {
                    $base = $bid->theme->current;
                    $param = $services->formatName('template.block--' . $base . '-' . $bundle . "-" . $mode_view . ".html.twig");
                }
            } else {
                $query = $this->getRequest()->query;
                if ($query->has('destination')) {
                    $options = UrlHelper::parse($query->get('destination'));
                    $url = Url::fromUri('internal:/' . $options['path'], $options);
                } else {
                    $url = Url::fromRoute('templating.manager');
                }
                $message = t('Template not find');
                \Drupal::messenger()->addMessage($message, 'error');
                // return new RedirectResponse($url->toString());
                return null;
            }
        }
        return $param;
    }
    /**
     * {@inheritdoc}
     */
    public function buildForm(array $form, FormStateInterface $form_state, $config_name = '')
    {

        $services = \Drupal::service('templating.manager');
        $query = $this->getRequest()->query;
        $config_name = $query->get('config');
        $config_name_contexual_link = $this->processContexualLiknk();
        if ($config_name_contexual_link) {
            $config_name = $config_name_contexual_link;
        }
        $path_file = DRUPAL_ROOT . '/config/templates/' . $config_name . '.yml';
        $str = " ( yml not exist )";
        if (file_exists($path_file)) {
            $str = " ( yml exist ) ";
        }
        $config = \Drupal::config($config_name);
        $output = $config->get('content');
        $status = ($config->get('status')) ? $config->get('status') : false;
        $css = ($config->get('css')) ? $config->get('css') : '';
        $js = ($config->get('js')) ? $config->get('js') : '';
        $name = $services->removePrefix($config_name);

        $form['help'] = [
            '#type' => 'item',
            '#markup' => 'EDIT BLOCK CONTENT ' . $config_name . $str,
        ];
        $form['tabs'] = array(
            '#type' => 'horizontal_tabs',
            '#tree' => true,
            '#prefix' => '<div id="unique-wrapper">',
            '#suffix' => '</div>',
        );
        $form['tabs']['content_html'] = array(
            '#type' => 'details',
            '#title' => t('HTML'),
            '#collapsible' => true,
            '#collapsed' => true,
        );
        $form['tabs']['content_css'] = array(
            '#type' => 'details',
            '#title' => t('CSS'),
            '#collapsible' => true,
            '#collapsed' => true,
        );
        $form['tabs']['content_js'] = array(
            '#type' => 'details',
            '#title' => t('JS'),
            '#collapsible' => true,
            '#collapsed' => true,
        );
        $form['tabs']['content_guide_twig'] = array(
            '#type' => 'details',
            '#title' => t('DEVELOPPEMENT GUIDE'),
            '#collapsible' => true,
            '#collapsed' => true,
        );
        $form['tabs']['content_preview'] = array(
            '#type' => 'details',
            '#title' => t('Run'),
            '#collapsible' => true,
            '#collapsed' => true,
        );
        $status_acer = \Drupal::moduleHandler()->moduleExists('ace_editor');
        if ($status_acer) {
            $form['tabs']['content_html']['html'] = [
                '#type' => 'text_format',
                '#format' => 'html_code',
                '#default_value' => $output,
            ];
            $form['tabs']['content_css']['css'] = [
                '#type' => 'text_format',
                '#format' => 'css_code',
                '#default_value' => $css,
            ];
            $form['tabs']['content_js']['js'] = [
                '#type' => 'text_format',
                '#format' => 'js_code',
                '#default_value' => $js,
            ];
            $config_settings = \Drupal::config("template_inline.settings");
            $twig_1 = $config_settings->get('content_lib');
            $twig_1 = $twig_1 . "<br/>" . $config_settings->get('guide_twig');
            $form['tabs']['content_guide_twig']['twig'] = [
                '#type' => 'item',
                '#markup' => $twig_1,
            ];
            $form['tabs']['content_preview']['preview'] = [
                '#type' => 'item',
                '#markup' => 'preview',
            ];
        }
        $form['#attached']['library'][] = 'field_group/formatter.horizontal_tabs';

        $form['config_name'] = array(
            '#type' => 'hidden',
            '#default_value' => $config_name,
        );
        $form['status'] = array(
            '#type' => 'checkbox',
            '#title' => t('Enable template inline'),
            '#default_value' => $status,
        );
        $form['actions'] = ['#type' => 'actions'];
        $form['actions']['submit'] = [
            '#type' => 'submit',
            '#value' => $this->t('Save Template'),
        ];
        $form['actions']['export'] = [
            '#type' => 'submit',
            '#value' => 'Export',
            '#submit' => [[$this, 'exportProcess']],
        ];
        //  $form['actions']['import'] = [
        //   '#type' => 'submit',
        //   '#value' => 'Import',
        //   '#submit' => [[$this, 'importProcess']],
        //  ];
        $form['actions']['cancel'] = array(
            '#type' => 'link',
            '#title' => $this->t('Back to Template list'),
            '#url' => $this->buildCancelLinkUrl(),
        );

        return $form;
    }
    public function importProcess(array &$form, FormStateInterface $form_state)
    {$query = $this->getRequest()->query;
        $config_name = $query->get('config');
        $values = $form_state->getValues();
        if (isset($values['config_name'])) {
            $config_name = $values['config_name'];
            $service = \Drupal::service('config_export_import.manager');

            $path_file = DRUPAL_ROOT . '/config/templates/' . $values['config_name'] . '.yml';
            $service->importConfig($path_file, true);
        }
    }
    public function exportProcess(array &$form, FormStateInterface $form_state)
    {$query = $this->getRequest()->query;
        $config_name = $query->get('config');
        $values = $form_state->getValues();
        if ($config_name && $values['tabs']) {
            $content = $values['tabs']['content_html']['html']['value'];
            $css = $values['tabs']['content_css']['css']['value'];
            $js = $values['tabs']['content_js']['js']['value'];

            $this->configFactory()->getEditable($config_name)
                ->set('content', $content)
                ->set('css', $css)
                ->set('js', $js)
                ->set('status', $values['status'])
                ->save();
            $this->messenger()->addMessage($this->t('Config ' . $config_name . ' update  was successfully'));
        }

        if (isset($values['config_name'])) {
            $config_name = $values['config_name'];
            $service = \Drupal::service('config_export_import.manager');
            $path_file = DRUPAL_ROOT . '/config/templates';
            $service->exportConfigByFullPath($config_name, $path_file);

        }
    }
    /**
     * {@inheritdoc}
     */
    public function validateForm(array &$form, FormStateInterface $form_state)
    {

    }

    /**
     * {@inheritdoc}
     */
    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $values = $form_state->getValues();
        $config_name = $values['config_name'];
        if ($config_name && $values['tabs']) {
            $content = $values['tabs']['content_html']['html']['value'];
            $css = $values['tabs']['content_css']['css']['value'];
            $js = $values['tabs']['content_js']['js']['value'];
            $this->configFactory()->getEditable($config_name)
                ->set('content', $content)
                ->set('css', $css)
                ->set('js', $js)
                ->set('status', $values['status'])
                ->save();
            $this->messenger()->addMessage($this->t('Config ' . $config_name . ' update  was successfully'));
            //drupal_flush_all_caches();
        }

    }

    private function buildPreview()
    {

        $query = $this->getRequest()->query;
        $config_name = $query->get('config');
        $config = \Drupal::config($config_name);
        $type = $config->get('type');
        $entityQuery = \Drupal::entityQuery('block_content');
        $entityQuery->condition('type', $type);
        $entityQuery->range(0, 1);
        $blocks = $entityQuery->execute();

        $url = Url::fromRoute('templating.preview_page_controller_preview',
            ["entitype" => "block_content", "id" => reset($blocks)]);
        return $url;
    }
    /**
     * Builds the cancel link url for the form.
     *
     * @return Url
     *   Cancel url
     */
    private function buildCancelLinkUrl()
    {
        $query = $this->getRequest()->query;
        if ($query->has('destination')) {
            $options = UrlHelper::parse($query->get('destination'));
            $url = Url::fromUri('internal:/' . $options['path'], $options);
        } else {
            $url = Url::fromRoute('templating.manager');
        }

        return $url;
    }

}
